c ------ Subroutine CH_INIT (idsym,idpts) ------------------------------
c Set collocation points and Chebyshev matrices for collocation method
c
c     Author: Thorwald Herbert, OSU, 1988
c
c     idsym <  0 interval -1, 1, polynomials Tk, k=1, ...   jx+nx
c           >= 0 interval  0, 1, polynomials Tk, k=1, ... 2*jx+nx+1
c     idpts != 0 Gauss-Lobatto points
c            = 0 Gauss points
c ----------------------------------------------------------------------

      subroutine chinit(idsym,idpts)
#include "common.ins"
      common /collot/ chx(0:jx),chtx(0:jx,0:2*jx+nx+1,0:nx)
     |                         ,chty(0:jx,0:2*jx+nx+1,0:nx)
      one=1
      pi=2*asin(one)
c
c collocation points x(j), T0=1, and T1=x
c

c provide for symmetry

      if (idsym.lt.0) then
         jj=jx
         kx=jx+nx
      else
         jj=2*jx+1
         kx=2*jx+nx+1
      endif

c distinguish Gauss and Gauss-Lobatto points

      if (idpts.ne.0) then
         dxs=sin(pi/jj)
         dxc=sqrt((1-dxs)*(1+dxs))
         xs=0
         chx(0)=1
      else
         xs=sin(pi/(2*jj+2))
         chx(0)=sqrt((1-xs)*(1+xs))
         dxs=2*xs*chx(0)
         dxc=sqrt((1-dxs)*(1+dxs))
      endif

c set xj, T0(xj) and T1(xj)

      chtx(0,0,0)=1
      chtx(0,1,0)=chx(0)
      do 1 j=1,jx
         chx(j)=dxc*chx(j-1)-dxs*xs
         xs    =dxs*chx(j-1)+dxc*xs
         chtx(j,0,0)=1
         chtx(j,1,0)=chx(j)
1     continue

c values chtx(j,k,0) of Tk(xj), k>1

      do 2 j=0,jx
      do 2 k=2,kx
      chtx(j,k,0)=2*chx(j)*chtx(j,k-1,0)-chtx(j,k-2,0)
2     continue

c values chtx(j,k,n) of the n-th derivative of Tk at xj

      do 3 j=0,jx
      do 3 n=1,nx
         x0=0
         x1=chtx(j,0,n-1)
         chtx(j,0,n)=0
         chtx(j,1,n)=x1
         do 3 k=2,kx
         x2=x1
         x1=x0+2*chtx(j,k-1,n-1)
         x0=x2
         chtx(j,k,n)=x1*k
3     continue
c
      return
      end
