c ------ Subroutine H_GLOBAL -------------------------------------------
c Set the matrices A and B of the homogeneous system and solve the
c homogeneous system (A-sigma*B) a = 0 for all eigenvalues
c
c     Author: Thorwald Herbert, OSU, 1988
c
c ----------------------------------------------------------------------

      subroutine hglobal(indt,indc)
#include "common.ins"
      common /collof/ chy(0:jx),chfy(0:jx,0:jx,0:nx,1:nvar)
      common /vector/ pvec(0:jx,1:nequ,0:nx,1:nvar,0:npar),
     |                rvec(0:jx,1:nequ,0:nx,1:nvar,2)
      common /glomat/ amat(0:jx,1:nequ,0:jx,1:nvar),
     |                bmat(0:jx,1:nequ,0:jx,1:nvar)
      common /params/ con(0:ncon),par(npar),dpar(npar,2),ipar(npar,2),
     |                itask
      common /spect/  er(isize),ei(isize),ar(isize),ai(isize),
     |                ir(isize),nev
      dimension bmatr(0:jx,1:nequ,0:jx,1:nvar),
     |          bmati(0:jx,1:nequ,0:jx,1:nvar),
     |          itmp(isize),tmp1(isize),tmp2(isize),tmp3(isize)
#ifdef CRAY
      complex pvec,rvec,uimag,zero,amat,bmat,zz
#else
      complex*16 pvec,rvec,uimag,zero,amat,bmat,zz
#endif

      zro=0
      one=1
      uimag=cmplx(zro,one)
      zero=cmplx(zro,zro)

c obtain coefficient matrices at collocation points for given sigma

      if (indt.ne.0) then
         print*,'H_GLOBAL: Eigenvalue appears nonlinear'
         stop
      endif
      if (indc.eq.0) then
         print*,'H_GLOBAL: Eigenvalue is not complex'
         stop
      endif
      call pvector
      call qvector

c set the matrices A and B

      do 1 j=0,jx
        do 1 i=1,nequ
          do 1 k=0,jx
            do 1 l=1,nvar
              amat(j,i,k,l)=zero
              bmat(j,i,k,l)=zero
              do 1 n=0,nx
              amat(j,i,k,l)=amat(j,i,k,l)+chfy(j,k,n,l)*pvec(j,i,n,l,0)
              bmat(j,i,k,l)=bmat(j,i,k,l)-chfy(j,k,n,l)*rvec(j,i,n,l,1)
1     continue

c multiply with inverse of A. result is in B

      call algc(amat,itmp,isize,isize,isize,1,zz)

c rearrange matrix

      do 2 j=0,jx
        do 2 i=1,nequ
          do 2 k=0,jx
            do 2 l=1,nvar
              bmatr(k,l,j,i)=bmat(j,i,k,l)
              bmati(k,l,j,i)=-bmat(j,i,k,l)*uimag
2     continue

c use eispack routine to obtain eigenvalues

      call cg(isize,isize,bmatr,bmati,er,ei,0,x,x,tmp1,tmp2,tmp3,ierr)
      if (ierr.ne.0) then
         print*,'H_GLOBAL: Eispack CG error, ierr=', ierr
      endif

c invert eigenvalues (exclude spurious values)

      do 3 j=1,isize
         x=er(j)**2+ei(j)**2
         if (x.lt.1e-20) then
            er(j)=-9.99999999e10
            er(j)=-9.99999999e10
         else
            er(j)= er(j)/x
            ei(j)=-ei(j)/x
         endif
3     continue

c sort eigenvalues by descending real part

      print 2000
2000  format(/'   No',12x,'Eigenvalue',15x,'No',12x,'Eigenvalue')
      write(iologo,2000)

      do 4 j=1,isize
      do 5 k=j+1,isize
      if (er(k).gt.er(j)) then
         x    =er(j)
         er(j)=er(k)
         er(k)=x
         x    =ei(j)
         ei(j)=ei(k)
         ei(k)=x
      endif
5     continue
      if (er(j).gt.-1e10) then
         ir(j)=j
         ar(j)=er(j)
         ai(j)=ei(j)
         nev=j
      endif
4     continue

c sort eigenvalues by descending imaginary part and print

      do 6 j=1,nev
      do 7 k=j+1,nev
      if (ai(k).gt.ai(j)) then
         i    =ir(j)
         ir(j)=ir(k)
         ir(k)=i
         x    =ar(j)
         ar(j)=ar(k)
         ar(k)=x
         x    =ai(j)
         ai(j)=ai(k)
         ai(k)=x
      endif
7     continue
      print '(2(i5,1p2e17.9))',j,er(j),ei(j),ir(j),ar(j),ai(j)
      write(iospec,'(2(i5,1p2e17.9))') j,er(j),ei(j),ir(j),ar(j),ai(j)
      write(iologo,'(2(i5,1p2e17.9))') j,er(j),ei(j),ir(j),ar(j),ai(j)
6     continue
      if (nev.ne.isize) then
         print*
         write(iospec,*)
         write(iologo,*)
         print*, isize-nev,' eigenvalues >1e10 excluded'
         write(iospec,*) isize-nev,' eigenvalue(s) >1e10 excluded'
         write(iologo,*) isize-nev,' eigenvalue(s) >1e10 excluded'
      endif
      close(iospec)
      return
      end

