c ------ Subroutine B_STATE --------------------------------------------
c Set the basic state for use in P_VECTOR, Q_VECTOR
c     The jx collocation points are in chy(0:jx)
c
c     Author: Thorwald Herbert, OSU, 1988
c
c ----------------------------------------------------------------------

      subroutine bstate
c **********************************************************************
c Perform 64-bit arithmetic
      implicit real*8 (a-h,o-z)

c Number of collocation points jx
      PARAMETER (JX= 30)

c Number of equations and variables
      PARAMETER (NEQU= 1)
      parameter (nvar=nequ,isize=(jx+1)*nequ)

c Highest derivative nx
      PARAMETER (NX= 4)

c Number of constants (or zero)
      PARAMETER (NCON= 0)

c Number of parameters
      PARAMETER (NPAR= 6)
c **********************************************************************
c Numbers of input/output units
      parameter (iodefs=99)
      parameter (iopars=98)
      parameter (iologi=97)
      parameter (iologo=96)
      parameter (iospec=95)
      parameter (iofunc=94)
      parameter (ioseri=93)
      parameter (iotabl=92)
      parameter (iocurv=91)
      PARAMETER (IOSTAT=80)
c **********************************************************************
c Arrays for basic state
      COMMON /MSTATE/ DISPL,UM(0:JX,0:2),RVM(0:JX,0:2)
c **********************************************************************

      common /collof/ chy(0:jx),chfy(0:jx,0:jx,0:nx,1:nvar)
      common /params/ con(0:ncon),par(npar),dpar(npar,2),ipar(npar,2),
     |                itask
      common /names/  conname(0:ncon),parname(npar),varname(nvar),
     |                problem
      character*8 conname,parname,varname
      character*80 problem
c
c      do 1 j=0,jx

c ***********************************************************************
c ***********************************************************************
CASE: Blasius Boundary-Layer Flow

c     Author: Thorwald Herbert, OSU, 1989

c     Solution of 2 f''' + ff'' = 0 with
c     f = f' = 0 at y = 0, f' -> 1 as y -> inf.
c     The (unusual) 2 is for consistency with the use of 
c     delta sub r = sqrt ( nu x / U ), Re = sqrt ( Re sub x ).

c     The equation is solved using Runge-Kutta-Gill integration
c     between collocation points.

c     The solution, once calculated, is stored in file 'Blasius' and
c     is recalculated automatically if relevant parameters change.
c     Otherwise, the solution is read from file 'Blasius'.
        
      dimension yy(3),ff(3),qq(3),fb(4)
      character*12 string
      
      open (iostat,file='Blasius',status='unknown')

c try to read mean flow from file

      rewind (iostat)
      read (iostat,1000,end=100,err=100) string,displ,k
1000  format (a12,8x,f10.7,5x,i3//)
      if (string.eq.'Blasius Flow'.and.k.eq.jx) then
         do 101 j=0,jx
            read (iostat,1001) yj,(um(j,n),n=0,2)
1001        format (3x,1p4e17.9)
            if (abs(yj-chy(j)).gt.1e-9*chy(j)) go to 100
101      continue
         read (iostat,1002) (yj,(rvm(j,n),n=0,2),j=0,jx)
1002     format (//3x,1p4e17.9/(3x,4e17.9))
      else

c initialize flow calculation

100      rewind (iostat)
         id=0
         yy(1)=.3320573362236681d0
         yy(2)=0
         yy(3)=0
         displ=0
         do 10 j=0,jx
            if (j.eq.0) then
               fb(1)=0
               fb(2)=0
            else 
               if (yy(1).gt.1e-16) then
                  x=chy(j-1)
                  dx=chy(j)-x
                  is=1+70*dx
                  h=dx/is

c runge-kutta by gills method

12                id=id+1
                  go to (1,4,5,3,7),id

1                 do 2 i=1,3
                     qq(i)=0
2                 continue
                  a=.5
                  go to 9

3                 a=1.707106781186548d0
4                 x=x+h/2
5                 do 6 i=1,3
                     yy(i)=yy(i)+a*(ff(i)*h-qq(i))
                     qq(i)=2*a*h*ff(i)+(1-3*a)*qq(i)
6                 continue
                  a=.292893218813452d0
9                 ff(1)=-yy(1)*yy(3)/2
                  ff(2)=yy(1)
                  ff(3)=yy(2)
                  go to 12

7                 do 8 i=1,3
                     yy(i)=yy(i)+h*ff(i)/6-qq(i)/3
8                 continue
                  id=0
                  is=is-1
                  if (is.gt.0) go to 12
                  displ=x-yy(3)
                  fb(1)=yy(3)
               else
                  fb(1)=chy(j)-displ
               endif
               fb(2)=yy(2)
            endif
            fb(3)=yy(1)
            fb(4)=-yy(1)*yy(3)/2
            um(j,0)=fb(2)
            um(j,1)=fb(3)
            um(j,2)=fb(4)
            rvm(j,0)=(fb(1)-chy(j)*fb(2))/2
            rvm(j,1)=-chy(j)*fb(3)/2
            rvm(j,2)=(-fb(3)-chy(j)*fb(4))/2
c           rvm(j,3)=-fb(4)+chy(j)*(fb(2)*fb(3)+fb(1)*fb(4))/4
10       continue

c write file 'Blasius'

         write (iostat,1003) displ,jx,(j,chy(j),(um(j,n),n=0,2),j=0,jx)
1003     format ('Blasius Flow. Displ=',f10.7,', jx=',i3//
     |          2x,'j',8x,'y',16x,2hum,15x,3hum',14x,4hum''/
     |          (i3,1p4e17.9))
         write (iostat,1005) (j,chy(j),(rvm(j,n),n=0,2),j=0,jx)
1005     format (/2x,'j',8x,'y',15x,5hRe*vm,12x,6hRe*vm',11x,7hRe*vm''/
     |          (i3,1p4e17.9))
         close (iostat)
      endif
c ***********************************************************************
c ***********************************************************************

c1     continue
      return
      end
