c ------ Subroutine NEXT_PT --------------------------------------------
c Arc length continuation with automatic step control to trace one real
c eigenvalue along a curve in a plane and another along a line
c
c     Author: Thorwald Herbert, OSU, 1989
c
c ----------------------------------------------------------------------

      subroutine nextpt(no,iter,radius,angle,dist)



      implicit real*8 (a-h,o-z)

      common /vcurve/ vini(3),vcon(3),veps(2),vgrid(2)
      common /pcurve/ var(3,3),unitx(3),unity(3),gap(3),
     *                dvarx,dvary,evarx,evary,
     *                one,pi,rad,div,ano,
     *                x0,y0,r,phi1,phi2,dphi,vx0,vy0,
     *                ino,icon
      common /scurve/ cavg,crad,cang

      dist=999
      if (no.lt.0) then
         ino=0
         icon=0
         no=0
      else if (icon.ne.0.and.iter.gt.0) then
         ino=ino+1
         ano=ano+1/div
         cavg=ano/ino
         no=ano
         i0=1+mod(ino,3)
         i1=1+mod(ino+1,3)
         i2=1+mod(ino+2,3)
         do 1 i=1,3
1        var(i,i2)=vcon(i)
         gap(i2)=1/div
         icon=0
         if (ino.eq.1) then
            vx0=vcon(1)
            vy0=vcon(2)
         endif
         if (no.ge.3) then
            dist=sqrt(((vcon(1)-vx0)/vgrid(1))**2
     |               +((vcon(2)-vy0)/vgrid(2))**2)/radius
         endif
      endif

c *** first point (given direction): initialize, find normal direction

      if (ino.eq.0) then
         ano=0
         if (icon.ne.0.and.iter.lt.0) then
            print*, 'No convergence for initial solution. Stop.'
            stop
         endif
         one=1
         div=1
         pi=2*asin(one)
         rad=pi/180
         crad=radius
         cang=angle

c     unit vectors - advance direction

         dvarx=cos(rad*angle)
         dvary=sin(rad*angle)

c     unit vectors - iteration direction

         evarx=-dvary
         evary=dvarx
         veps(1)=evarx
         veps(2)=evary
      endif

c *** second point: advance in initial direction

      if (ino.eq.1) then
         if (icon.eq.0) then
            if (div.gt.1) div=div/2
         else
            if (div.gt.1025) then
               print*, 'Continuation fails for radius/2**10. Stop.'
               stop
            endif
            div=2*div
         endif
         veps(1)=evarx
         veps(2)=evary
         vini(1)=var(1,i2)+vgrid(1)*dvarx*radius/div
         vini(2)=var(2,i2)+vgrid(2)*dvary*radius/div
         vini(3)=var(3,i2)
         crad=radius/div
      endif

c *** third point: update advance direction (straight line)

      if (ino.eq.2) then
         if (icon.eq.0) then
            if (div.gt.1) div=div/2

c     unit vectors - advance direction

            dvarx=(var(1,i2)-var(1,i1))/vgrid(1)
            dvary=(var(2,i2)-var(2,i1))/vgrid(2)
            dlen=sqrt(dvarx**2+dvary**2)
            dvarx=dvarx/dlen
            dvary=dvary/dlen
            cang=atan2(dvary,dvarx)/rad

c     unit vectors - iteration direction

            evarx=-dvary
            evary=dvarx
         else
            if (div.gt.1025) then
               print*, 'Continuation fails for radius/2**10. Stop.'
               stop
            endif
            div=2*div
         endif
         veps(1)=evarx
         veps(2)=evary
         vini(1)=var(1,i2)+vgrid(1)*dvarx*radius/div
         vini(2)=var(2,i2)+vgrid(2)*dvary*radius/div
         vini(3)=var(3,i2)+(var(3,i2)-var(3,i1))/gap(i2)/div
         crad=radius/div
      endif

c *** fourth and higher points

      if (ino.ge.3) then
         if (icon.eq.0) then
            if (div.gt.1) div=div/2

c     circle through last three points
            do 2 i=1,3
            unitx(i)=var(1,i)/vgrid(1)
2           unity(i)=var(2,i)/vgrid(2)
            a11=2*(unitx(i2)-unitx(i1))
            a21=2*(unitx(i1)-unitx(i0))
            a12=2*(unity(i2)-unity(i1))
            a22=2*(unity(i1)-unity(i0))
            det=a11*a22-a12*a21
            if (det.ne.0) then
               b1=unitx(i2)**2-unitx(i1)**2
     *           +unity(i2)**2-unity(i1)**2
               b2=unitx(i1)**2-unitx(i0)**2
     *           +unity(i1)**2-unity(i0)**2
               x0=(a22*b1-a12*b2)/det
               y0=(a11*b2-a21*b1)/det
               r=sqrt((unitx(i2)-x0)**2+(unity(i2)-y0)**2)
               phi1=atan2((unity(i1)-y0),(unitx(i1)-x0))
               phi2=atan2((unity(i2)-y0),(unitx(i2)-x0))
               dphi=phi2-phi1
               if (dphi.lt.-pi) dphi=dphi+2*pi
               if (dphi.gt.pi) dphi=dphi-2*pi
               dphi=sign(radius/r,dphi)
3              if (abs(dphi)/div.gt.pi/18) then
                  div=2*div
                  go to 3
               endif
            endif
         else
            if (div.gt.1025) then
               print*, 'Continuation fails for radius/2**10. Stop.'
               stop
            endif
            div=2*div
         endif

c     unit vectors - advance direction

         if (det.eq.0) then
            veps(1)=evarx
            veps(2)=evary
            vini(1)=var(1,i2)+vgrid(1)*dvarx*radius/div
            vini(2)=var(2,i2)+vgrid(2)*dvary*radius/div
         else
            phi=phi2+dphi/div
            dvarx=-sin(phi)*sign(one,dphi)
            dvary=cos(phi)*sign(one,dphi)
            cang=atan2(dvary,dvarx)/rad

c     unit vectors - iteration direction

            evarx=-dvary
            evary=dvarx

c     initial values (on circle)

            veps(1)=evarx
            veps(2)=evary
            vini(2)=vgrid(2)*(y0+r*sin(phi))
            vini(1)=vgrid(1)*(x0+r*cos(phi))
         endif
         xa=((var(3,i2)-var(3,i1))/gap(i2)
     |      -(var(3,i1)-var(3,i0))/gap(i1))/(gap(i2)+gap(i1))
         vini(3)=var(3,i2)+(((var(3,i2)-var(3,i1))/gap(i2)+xa*gap(i2))
     |          +xa/div)/div
         crad=radius/div
      endif
      icon=1
      return
      end
