c ------ Subroutine P_VECTOR -------------------------------------------
c Set the coefficient matrix P at collocation points for given sigma
c
c     Author: Thorwald Herbert, OSU, 1988
c
c ----------------------------------------------------------------------

      subroutine pvector
c **********************************************************************
c Perform 64-bit arithmetic
      implicit real*8 (a-h,o-z)

c Number of collocation points jx
      PARAMETER (JX= 30)

c Number of equations and variables
      PARAMETER (NEQU= 1)
      parameter (nvar=nequ,isize=(jx+1)*nequ)

c Highest derivative nx
      PARAMETER (NX= 4)

c Number of constants (or zero)
      PARAMETER (NCON= 0)

c Number of parameters
      PARAMETER (NPAR= 6)
c **********************************************************************
c Numbers of input/output units
      parameter (iodefs=99)
      parameter (iopars=98)
      parameter (iologi=97)
      parameter (iologo=96)
      parameter (iospec=95)
      parameter (iofunc=94)
      parameter (ioseri=93)
      parameter (iotabl=92)
      parameter (iocurv=91)
      PARAMETER (IOSTAT=80)
c **********************************************************************
c Arrays for basic state
      COMMON /MSTATE/ DISPL,UM(0:JX,0:2),RVM(0:JX,0:2)
c **********************************************************************

      common /collof/ chy(0:jx),chfy(0:jx,0:jx,0:nx,1:nvar)
      common /vector/ pvec(0:jx,1:nequ,0:nx,1:nvar,0:npar),
     |                rvec(0:jx,1:nequ,0:nx,1:nvar,2)
      common /params/ con(0:ncon),par(npar),dpar(npar,2),ipar(npar,2),
     |                itask
      common /names/  conname(0:ncon),parname(npar),varname(nvar),
     |                problem
      character*8 conname,parname,varname
      character*80 problem
      complex*16 pvec,rvec,uimag,zero
c **********************************************************************
c ***********************************************************
      COMPLEX*16 ALPHA,OMEGA,GAMMA
c ***********************************************************
c **********************************************************************
      zro=0
      one=1
      uimag=cmplx(zro,one)
      zero=cmplx(zro,zro)

c reset coefficient matrices to zero

      do 1 j=0,jx
        do 1 i=1,nequ
           do 1 n=0,nx
              do 1 l=1,nvar
                 pvec(j,i,n,l,0)=zero
1     continue

c set coefficient matrix at collocation points for given parameters

      ideriv=0
      do 2 j=0,jx

c **********************************************************************
c **********************************************************************
c pvec(j,i,n,l,m)  with j = collocation point (set by program)
c                       i = i-th equation
c                       n = n-th derivative of
c                       l = l-th variable
c                       m = derivative wrto m-th parameter
                     
c CASE: Orr-Sommerfeld Equation for U(y), V=W=0

c gamma=alpha^2+beta^2
c [(D^2-gamma-i*alpha*Re*U-Re*sigma)(D^2-gamma)+i*alpha*Re] v = 0

      RE   =      PAR(1)
      ALPHA=CMPLX(PAR(2),PAR(3))
      BETA =      PAR(4)
      OMEGA=CMPLX(PAR(5),PAR(6))
      GAMMA=ALPHA**2+BETA**2

      if (ideriv.eq.0) then

         PVEC(J,1,4,1,0)=1
         PVEC(J,1,2,1,0)=-2*GAMMA-UIMAG*RE*(ALPHA*UM(J,0)-OMEGA)
         PVEC(J,1,0,1,0)=GAMMA*(GAMMA+UIMAG*RE*(ALPHA*UM(J,0)-OMEGA))
     |                  +RE*UIMAG*ALPHA*UM(J,2)
      else

c derivative d / d(Re)

         PVEC(J,1,2,1,1)=-UIMAG*(ALPHA*UM(J,0)-OMEGA)
         PVEC(J,1,0,1,1)=UIMAG*(GAMMA*(ALPHA*UM(J,0)-OMEGA)
     |                  +ALPHA*UM(J,2))
    
c derivative d / d(alpha) = d / d(alphar)

         PVEC(J,1,2,1,2)=-4*ALPHA-UIMAG*RE*UM(J,0)
         PVEC(J,1,0,1,2)=GAMMA*(4*ALPHA+UIMAG*RE*UM(J,0))+UIMAG*RE
     |                  *(2*ALPHA*(ALPHA*UM(J,0)-OMEGA)+UM(J,2))

c derivative d / d(alphai)

         PVEC(J,1,2,1,3)=UIMAG*PVEC(J,1,2,1,2)
         PVEC(J,1,0,1,3)=UIMAG*PVEC(J,1,0,1,2)

c derivative d / d(beta)

         PVEC(J,1,2,1,4)=-4*BETA
         PVEC(J,1,0,1,4)=BETA*(4*GAMMA+2*UIMAG*RE*(ALPHA*UM(J,0)
     |                  -OMEGA))

c derivative d / d(omega) = d / d(omegar)

         PVEC(J,1,2,1,5)=UIMAG*RE
         PVEC(J,1,0,1,5)=-GAMMA*UIMAG*RE

c derivative d / d(omegai)

         PVEC(J,1,2,1,6)=UIMAG*PVEC(J,1,2,1,5)
         PVEC(J,1,0,1,6)=UIMAG*PVEC(J,1,0,1,5)

      endif

c **********************************************************************
c **********************************************************************

2     continue
      return
      end
