C*ALGC* SOLVE A COMPLEX ALGEBRAIC SYSTEM ******************
C
C       Author: Thorwald Herbert, VPI & SU
C
C       Q = matrix, dimensioned for maximum NX rows
C       IQ= integer array of maximum dimension NX
C       NX= maximum dimension of matrix
C       N = actual dimension of matrix, N.le.NX
C       NR= number of right hand sides 
C       ID= 1: reduce, solve for NR right hand sides
C       ID= 0, NR > 0: solve for NR new right hand sides
C       ID= 0, NR = 0: Wielandt correction step
C       ID=-1: reduce, first Wielandt step
C       E = Wielandt correction
C
      SUBROUTINE ALGC(Q,IQ,NX,N,NR,ID,E)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION Q(2,NX,1),IQ(1),E(2)
C
      M=N+NR
      M1=N-1
      N1=N+1
      IF (ID.EQ.0) GO TO 10
      DO 1 K=1,M1
      XX=0
      KX=K
      DO 2 KK=K,N
      X=Q(1,KK,K)**2+Q(2,KK,K)**2
      IF (X.LE.XX) GO TO 2
      XX=X
      KX=KK
2     CONTINUE
      IQ(K)=KX
      IF (KX.EQ.K) GO TO 3
      DO 4 JJ=K,M
      DO 4 I=1,2
      Z=Q(I,K,JJ)
      Q(I,K,JJ)=Q(I,KX,JJ)
4     Q(I,KX,JJ)=Z
3     CONTINUE
      K1=K+1
      DO 1 KK=K1,N
      XX=Q(1,K,K)**2+Q(2,K,K)**2
      IF (XX.LE.0) GO TO 1
      X=        (Q(2,KK,K)*Q(1,K,K)-Q(1,KK,K)*Q(2,K,K))/XX
      Q(1,KK,K)=(Q(1,KK,K)*Q(1,K,K)+Q(2,KK,K)*Q(2,K,K))/XX
      Q(2,KK,K)=X
      DO 5 JJ=K1,M
      X=         Q(2,KK,JJ)-Q(2,KK,K)*Q(1,K,JJ)-Q(1,KK,K)*Q(2,K,JJ)
      Q(1,KK,JJ)=Q(1,KK,JJ)-Q(1,KK,K)*Q(1,K,JJ)+Q(2,KK,K)*Q(2,K,JJ)
5     Q(2,KK,JJ)=X
1     CONTINUE
      IF (ID.GT.0) GO TO 12
C
      Q(1,N,N1)=1
      Q(2,N,N1)=0
      DO 6 KK=1,M1
      K=N-KK
      Q(1,K,N1)=-Q(1,K,N)
      Q(2,K,N1)=-Q(2,K,N)
      IF (K.EQ.M1) GO TO 8
      K1=K+1
      DO 7 L=K1,M1
      X=        Q(2,K,N1)-Q(2,K,L)*Q(1,L,N1)-Q(1,K,L)*Q(2,L,N1)
      Q(1,K,N1)=Q(1,K,N1)-Q(1,K,L)*Q(1,L,N1)+Q(2,K,L)*Q(2,L,N1)
7     Q(2,K,N1)=X
8     XX=Q(1,K,K)**2+Q(2,K,K)**2
      X=        (Q(2,K,N1)*Q(1,K,K)-Q(1,K,N1)*Q(2,K,K))/XX
      Q(1,K,N1)=(Q(1,K,N1)*Q(1,K,K)+Q(2,K,N1)*Q(2,K,K))/XX
6     Q(2,K,N1)=X
      RETURN
C
10    IF (NR.LE.0) M=N1
      DO 11 L=N1,M
      DO 11 K=1,M1
      KX=IQ(K)
      X=Q(1,K,L)
      Q(1,K,L)=Q(1,KX,L)
      Q(1,KX,L)=X
      X=Q(2,K,L)
      Q(2,K,L)=Q(2,KX,L)
      Q(2,KX,L)=X
      K1=K+1
      DO 11 KK=K1,N
      X=        Q(2,KK,L)-Q(2,KK,K)*Q(1,K,L)-Q(1,KK,K)*Q(2,K,L)
      Q(1,KK,L)=Q(1,KK,L)-Q(1,KK,K)*Q(1,K,L)+Q(2,KK,K)*Q(2,K,L)
11    Q(2,KK,L)=X
      IF (NR.GT.0) GO TO 12
C
      XX=Q(1,N,M)**2+Q(2,N,M)**2
      E(1)=(Q(1,N,N)*Q(1,N,M)+Q(2,N,N)*Q(2,N,M))/XX
      E(2)=(Q(2,N,N)*Q(1,N,M)-Q(1,N,N)*Q(2,N,M))/XX
      DO 13 K=1,N
      X=       E(2)*Q(1,K,M)+E(1)*Q(2,K,M)
      Q(1,K,M)=E(1)*Q(1,K,M)-E(2)*Q(2,K,M)
13    Q(2,K,M)=X
C
12    IF (M.LE.N) RETURN
      DO 14 KK=1,N
      K=N1-KK
      IF (K.EQ.N) GO TO 15
      K1=K+1
      DO 16 L=N1,M
      DO 16 LL=K1,N
      X=       Q(2,K,L)-Q(2,K,LL)*Q(1,LL,L)-Q(1,K,LL)*Q(2,LL,L)
      Q(1,K,L)=Q(1,K,L)-Q(1,K,LL)*Q(1,LL,L)+Q(2,K,LL)*Q(2,LL,L)
16    Q(2,K,L)=X
15    DO 14 L=N1,M
      XX=Q(1,K,K)**2+Q(2,K,K)**2
      X=       (Q(2,K,L)*Q(1,K,K)-Q(1,K,L)*Q(2,K,K))/XX
      Q(1,K,L)=(Q(1,K,L)*Q(1,K,K)+Q(2,K,L)*Q(2,K,K))/XX
14    Q(2,K,L)=X
      RETURN
      END
 
