c ------ Subroutine CH_TtoF (idvar,idsym,idbc) -------------------------
c Derive functions F from T that satisfy the boundary conditions
c
c     Author: Thorwald Herbert, OSU, 1988
c
c     idvar : index of the independent variable 1<=idvar<=nvar
c     idsym < 0 interval -1, 1, polynomials Tk, k=0 (1) jx+nx
c           = 0 interval  0, 1, polynomials Tk, k=0 (2) 2*jx+nx
c           > 0 interval  0, 1, polynomials Tk, k=1 (2) 2*jx+nx+1
c     idbc  : index indicating type of boundary conditions
c           = 0 none, use plain polynomials
c           = 1 f(a)=f(b)=0
c           = 2 f'(a)=f'(b)=0
c           = 3 f(a)=f(b)=f'(a)=f'(b)=0
c
c ----------------------------------------------------------------------

      subroutine chttof(idvar,idsym,idbc)
#include "common.ins"
      common /collot/ chx(0:jx),chtx(0:jx,0:2*jx+nx+1,0:nx)
     |                         ,chty(0:jx,0:2*jx+nx+1,0:nx)
      common /collof/ chy(0:jx),chfy(0:jx,0:jx,0:nx,1:nvar)

c check for valid variable

      if (idvar.lt.1.or.idvar.gt.nvar) then
         print*,'CH_TtoF: Illegal variable'
         stop
      endif

c provide for symmetry

      if (idsym.lt.0) then
         k0=0
         k1=1
      else if (idsym.eq.0) then
         k0=0
         k1=2
      else
         k0=1
         k1=2
      endif

c implement boundary conditions

      do 1 n=0,nx
         do 1 j=0,jx
            do 1 k=0,jx
               l=k0+k*k1
               go to (10,11,12,13),idbc+1
               print*,'CH_TtoF: Illegal boundary condition'
               stop

c no boundary condition

10             chfy(j,k,n,idvar)=chty(j,l,n)
               go to 1

c boundary condition 1: f(a)=f(b)=0

11             chfy(j,k,n,idvar)=chty(j,l,n)-chty(j,l+2,n)
               go to 1

c boundary condition 2: f'(a)=f'(b)=0

12             chfy(j,k,n,idvar)=((l+2)**2*chty(j,l,n)
     |                               -l**2*chty(j,l+2,n))/(l+1)**2
               go to 1

c boundary condition 3: f(a)=f(b)=f'(a)=f'(b)=0

13             chfy(j,k,n,idvar)=((l+3)*chty(j,l,n)
     |                           +(l+1)*chty(j,l+4,n))/(2*l+4)
     |                                 -chty(j,l+2,n)

1     continue
      return
      end
